#ifndef AUBO_SDK_RTDE_H
#define AUBO_SDK_RTDE_H

#include <string>
#include <vector>
#include <functional>
#include <memory>
#include <map>

#include <aubo/type_def.h>
#include <aubo/global_config.h>

namespace arcs {
namespace aubo_sdk {

class RtdeClient;

/// 向输出数据中增加
class ARCS_ABI_EXPORT OutputBuilder
{
public:
    OutputBuilder();
    ~OutputBuilder();

    OutputBuilder &push(int val);
    OutputBuilder &push(double val);
    OutputBuilder &push(const std::vector<double> &val);
    OutputBuilder &push(const std::vector<int> &val);
    OutputBuilder &push(const std::string &val);
    OutputBuilder &push(char val);
    OutputBuilder &push(const common_interface::RtdeRecipe &val);

private:
    friend RtdeClient;
    class Impl;
    Impl *impl;
};

/// 解析输入
class ARCS_ABI_EXPORT InputParser
{
public:
    InputParser();
    ~InputParser();

    int popInt32();
    int64_t popInt64();
    double popDouble();
    char popChar();
    std::vector<double> popVectorDouble();
    std::vector<common_interface::JointStateType> popVectorJointStateType();
    common_interface::RobotModeType popRobotModeType();
    common_interface::SafetyModeType popSafetyModeType();
    common_interface::RuntimeState popRuntimeState();
    common_interface::RobotMsgVector popRobotMsgVector();

private:
    friend RtdeClient;
    class Impl;
    Impl *impl;
};

/// RTDE客户端
class ARCS_ABI_EXPORT RtdeClient
{
public:
    RtdeClient();
    ~RtdeClient();

    // 0: LOGLEVEL_FATAL
    // 1: LOGLEVEL_ERROR
    // 2: LOGLEVEL_WARNING
    // 3: LOGLEVEL_INFO
    // 4: LOGLEVEL_DEBUG
    // 5: LOGLEVEL_BACKTRACE
    void setLogHandler(
        std::function<void(int /*level*/, const char * /*filename*/,
                           int /*line*/, const std::string & /*message*/)>
            handler);

    /// 连接到服务器
    int connect(const std::string &ip, int port);

    /// socket 是否已连接
    bool hasConnected() const;
    bool hasConnected1(std::function<void(bool)> callback);

    /// 登录
    int login(const std::string &usrname, const std::string &passwd);

    /// 是否已经登录
    bool hasLogined();

    /// 登出
    int logout();

    /// 断开连接
    int disconnect();

    /// 获取协议版本号
    int getProtocolVersion();

    /// 获取输入列表
    std::map<std::string, int> getInputMaps();

    /// 获取输出列表
    std::map<std::string, int> getOutputMaps();

    /**
     * @brief setTopic
     * @param is
     * @param names
     * @param freq
     * @return chanel -1 表示失败 0~99 成功
     */
    int setTopic(bool to_server, const std::vector<std::string> &names,
                 double freq, int expected_chanel);

    /// 取消订阅
    int removeTopic(bool to_server, int chanel);

    /// 获取已注册的输入菜单
    std::unordered_map<int, common_interface::RtdeRecipe>
    getRegisteredInputRecipe();

    /// 获取已注册的输出菜单
    std::unordered_map<int, common_interface::RtdeRecipe>
    getRegisteredOutputRecipe();

    /**
     * 订阅 subscribe from output
     *
     */
    int subscribe(int chanel, std::function<void(InputParser &)> callback);

    /// 发布 publish to input
    int publish(int chanel, std::function<void(OutputBuilder &)> callback);

private:
    class Impl;
    Impl *impl;
};
using RtdeClientPtr = std::shared_ptr<RtdeClient>;

} // namespace aubo_sdk
} // namespace arcs
#endif
