#ifndef AUBO_SCOPE_ROBOT_PROXY_H
#define AUBO_SCOPE_ROBOT_PROXY_H

#include <memory>
#include <vector>
#include <string>

#include <QObject>
#include <QMutex>

#include <aubo/aubo_api.h>

#include <robot_proxy/realtime_robot_state.h>

namespace arcs {
namespace aubo_sdk {
class RtdeClient;
class ScriptClient;
class RpcClient;
using RtdeClientPtr = std::shared_ptr<RtdeClient>;
using ScriptClientPtr = std::shared_ptr<ScriptClient>;
using RpcClientPtr = std::shared_ptr<RpcClient>;

using arcs::common_interface::ForceControlPtr;
using arcs::common_interface::IoControlPtr;
using arcs::common_interface::MotionControlPtr;
using arcs::common_interface::RegisterControlPtr;
using arcs::common_interface::RobotAlgorithmPtr;
using arcs::common_interface::RobotConfigPtr;
using arcs::common_interface::RobotInterfacePtr;
using arcs::common_interface::RobotManagePtr;
using arcs::common_interface::RobotStatePtr;
using arcs::common_interface::RuntimeMachinePtr;
using arcs::common_interface::SyncMovePtr;
using arcs::common_interface::SystemInfoPtr;
using arcs::common_interface::TracePtr;

using RobotInterface = common_interface::RobotInterface;

/**
 * 单个机器人代理接口类
 */
class ARCS_ABI_EXPORT RobotProxy : public QObject
{
    Q_OBJECT

public:
    RobotProxy(QObject *parent = NULL);
    ~RobotProxy();

    /// 获取当前机器人的名称
    QString getRobotName() const;

    std::vector<std::string> getRobotNames();

    /// 连接到当前机器人
    int connect(const QString &ip, int port);

    /// 连接到当前机器人
    int nonblock_connect(const QString &ip, int port,
                         std::function<void(bool)> cb);

    /// 登录到当前机器人
    int login(const QString &usrname, const QString &passwd);

    /// 断开与机器人的连接，如果不指定名称，则断开当前选中的机器人
    int disconnectFromServer();

    /// 机器人是否已经连接，如果不指定名称，则断开当前选中的机器人
    bool hasConnected();

    /// 机器人是否已经登录，如果不指定名称，则断开当前选中的机器人
    bool hasLogined();

    /// 发送脚本程序到当前机器人
    int sendScript(const std::string &script);

    std::unordered_map<std::string, std::vector<double>> getDH();

    SystemInfoPtr getSystemInfo();
    RuntimeMachinePtr getRuntimeMachine();
    RegisterControlPtr getRegisterControl();

    /// 获取RobotConfig接口
    RobotConfigPtr getRobotConfig();

    /// 获取运动规划接口
    MotionControlPtr getMotionControl();

    /// 获取力控接口
    ForceControlPtr getForceControl();

    /// 获取IO控制的接口
    IoControlPtr getIoControl();

    /// 获取同步运动接口
    SyncMovePtr getSyncMove();

    /// 获取机器人实用算法接口
    RobotAlgorithmPtr getRobotAlgorithm();

    /// 获取机器人管理接口(上电、启动、停止等)
    RobotManagePtr getRobotManage();

    /// 获取机器人状态接口
    RobotStatePtr getRobotState();

    RealtimeRobotStatePtr getRealTimeState();

    /// 获取告警信息接口
    TracePtr getTrace();

    /// 根据机器人名字获取机器人接口
    RobotInterfacePtr getRobotInterface(const std::string &name);

    /// 切换机器人
    int selectRobot(int index);

signals:
    void connected(const QString &ip, int port);
    void disconnected();

    void popup(int level, const QString &title, const QString &msg, int mode);
    void robotMessageRecieved(QString src, int level, int code,
                              const QString &msg);

    void operationalModeChanged(OperationalModeType mode);
    void freedriveEnabled(bool enabled);

    void scriptError(const QString &);
    void variableUpdate(const QString &, const QString &);

private:
    RobotInterfacePtr currentRobotRpcInterface();

private:
    QString name_;
    std::vector<std::string> robot_names_;

    std::vector<double> a_, d_, alpha_, delta_, beta_;

    std::string ip_;
    int port_;

    aubo_sdk::RpcClientPtr rpc_client_{ nullptr };
    aubo_sdk::RtdeClientPtr rtde_client_{ nullptr };
    aubo_sdk::ScriptClientPtr script_client_{ nullptr };

    RealtimeRobotStatePtr rt_state_{ nullptr };

    bool initiallized_{ false };
};
using RobotProxyPtr = std::shared_ptr<RobotProxy>;

} // namespace aubo_sdk
} // namespace arcs
Q_DECLARE_METATYPE(arcs::aubo_sdk::RobotProxyPtr)
#endif
