#include "aubo_sdk/rpc.h"
#include "aubo_sdk/rtde.h"
#include <aubo/error_stack/error_stack.h>
#include <math.h>
using namespace arcs::common_interface;
using namespace arcs::aubo_sdk;

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif
std::vector<double> waypoint1; // 圆弧轨迹的起始路点（位姿）
std::vector<double> waypoint2; // 圆弧轨迹的中间路点（位姿）
std::vector<double> waypoint3; // 圆弧轨迹的终止路点（位姿）

// 获取圆弧轨迹的路点
// 依据 (x-0.12294)^2 + (y-0.54855)^2 + (z-0.26319)^2 = 0.2^2 来计算
void get_circle_waypoint()
{
    double z1 = 0.26319;
    double y1 = 0.7;
    double x1 = sqrt(pow(0.2, 2) - pow(y1 - 0.54855, 2) - pow(z1 - 0.26319, 2));
    waypoint1 = { x1, y1, z1, 3.14, 0.00, 3.14 };

    double z2 = 0.26319;
    double y2 = 0.55;
    double x2 = sqrt(pow(0.2, 2) - pow(y2 - 0.54855, 2) - pow(z2 - 0.26319, 2));
    waypoint2 = { x2, y2, z2, 3.14, 0.00, 3.14 };

    double z3 = 0.26319;
    double y3 = 0.35;
    double x3 = sqrt(pow(0.2, 2) - pow(y3 - 0.54855, 2) - pow(z3 - 0.26319, 2));
    waypoint3 = { x3, y3, z3, 3.14, 0.00, 3.14 };
}

// 实现阻塞功能: 当机械臂运动到目标路点时，程序再往下执行
void waitArrivel(RobotInterfacePtr impl)
{
    while (impl->getMotionControl()->getExecId() == -1) {
        std::this_thread::sleep_for(std::chrono::milliseconds(50));
    }
    auto id = impl->getMotionControl()->getExecId();
    while (1) {
        auto id1 = impl->getMotionControl()->getExecId();
        if (id != id1) {
            break;
        }
        std::this_thread::sleep_for(std::chrono::milliseconds(50));
    }
}

// 圆弧运动
void exampleMoveC(RpcClientPtr cli)
{
    int ret = 0;
    // 获取圆弧轨迹的路点
    get_circle_waypoint();
    // 接口调用: 获取机器人的名字
    auto robot_name = cli->getRobotNames().front();
    for (int i = 0; i < 3; i++) {
        // 接口调用: 直线运动到路点 waypoint1
        cli->getRobotInterface(robot_name)
            ->getMotionControl()
            ->moveLine(waypoint1, 180 * (M_PI / 180), 1000000 * (M_PI / 180), 0,
                       0);

        waitArrivel(cli->getRobotInterface(robot_name));
        // 接口调用: 圆弧运动
        ret = cli->getRobotInterface(robot_name)
                  ->getMotionControl()
                  ->moveCircle(waypoint2, waypoint3, 180 * (M_PI / 180),
                               1000000 * (M_PI / 180), 0, 0);
        if (ret < 0) {
            break;
        }

        waitArrivel(cli->getRobotInterface(robot_name));
    }

    if (ret == 0) {
        std::cout << "圆弧运动成功！" << std::endl;
    } else {
        std::cerr << "圆弧运动失败！错误码： " << ret << std::endl;
    }
}

// 高级圆弧或者圆周运动
void exampleMoveC2(RpcClientPtr cli)
{
    // 获取圆弧轨迹的路点
    get_circle_waypoint();
    // 接口调用: 获取机器人的名字
    auto robot_name = cli->getRobotNames().front();
    // 接口调用: 直线运动到路点 waypoint1
    cli->getRobotInterface(robot_name)
        ->getMotionControl()
        ->moveLine(waypoint1, 180 * (M_PI / 180), 1000000 * (M_PI / 180), 0, 0);
    std::this_thread::sleep_for(std::chrono::seconds(1));

    CircleParameters param;
    param.pose_via = waypoint2;
    param.pose_to = waypoint3;
    param.a = 180 * (M_PI / 180);
    param.v = 1000000 * (M_PI / 180);
    param.loop_times = 2;
    // 接口调用: 高级圆弧或者圆周运动
    int ret = cli->getRobotInterface(robot_name)
                  ->getMotionControl()
                  ->moveCircle2(param);

    if (ret == 0) {
        std::cout << "圆弧运动成功！" << std::endl;
    } else {
        std::cerr << "圆弧运动失败！错误码： " << ret << std::endl;
    }
}

void printlog(int level, const char *source, int code)
{
    static const char *level_names[] = { "Critical", "Error", "Warning",
                                         "Info",     "Debug", "BackTrace" };
    fprintf(stderr, "[%s] %s - %d %s\n", level_names[level], source, code,
            arcs::error_stack::errorCode2Str(code));
}

/**
 * 功能: 机械臂圆弧运动
 * 步骤:
 * 第一步: 连接 RPC 服务、机械臂登录
 * 第二步: 圆弧运动
 * 第三步: 高级圆弧或者圆周运动
 */
#define LOCAL_IP "127.0.0.1"

int main(int argc, char **argv)
{
    auto cli = std::make_shared<RpcClient>();
    // 接口调用: 连接到 RPC 服务
    cli->connect(LOCAL_IP, 30004);
    // 接口调用: 登录
    cli->login("aubo", "123456");

    auto rtde_cli = std::make_shared<RtdeClient>();
    // 接口调用: 连接到 RTDE 服务
    rtde_cli->connect(LOCAL_IP, 30010);
    // 接口调用: 登录
    rtde_cli->login("aubo", "123456");

    int topic = rtde_cli->setTopic(false, { "R1_message" }, 200, 0);
    if (topic < 0) {
        std::cout << "Set topic fail!" << std::endl;
        return -1;
    }

    rtde_cli->subscribe(topic, [](InputParser &parser) {
        arcs::common_interface::RobotMsgVector msgs;
        msgs = parser.popRobotMsgVector();
        for (size_t i = 0; i < msgs.size(); i++) {
            auto &msg = msgs[i];
            printlog(msg.level, msg.source.c_str(), msg.code);
        }
    });

    // 圆弧运动
    exampleMoveC(cli);

    // 高级圆弧或者圆周运动
    exampleMoveC2(cli);

    rtde_cli->removeTopic(false, topic);
    rtde_cli->logout();
    rtde_cli->disconnect();
    return 0;
}
