#include <cstring>
#include <fstream>
#include <math.h>
#include <unistd.h>

#include "aubo_sdk/rpc.h"
#include "aubo_sdk/rtde.h"
#include <aubo/error_stack/error_stack.h>
using namespace arcs::aubo_sdk;
using namespace arcs::common_interface;

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

void waitArrivel(RobotInterfacePtr impl)
{
    while (impl->getMotionControl()->getExecId() == -1) {
        std::this_thread::sleep_for(std::chrono::milliseconds(50));
    }
    auto id = impl->getMotionControl()->getExecId();
    while (1) {
        auto id1 = impl->getMotionControl()->getExecId();
        if (id != id1) {
            break;
        }
        std::this_thread::sleep_for(std::chrono::milliseconds(50));
    }
}

template <typename T>
inline std::ostream &operator<<(std::ostream &os, const std::vector<T> &list)
{
    for (size_t i = 0; i < list.size(); i++) {
        os << list.at(i);
        if (i != (list.size() - 1)) {
            os << ",";
        }
    }
    return os;
}

class TrajectoryIo
{
public:
    TrajectoryIo(const char *filename)
    {
        input_file_.open(filename, std::ios::in);
    }
    ~TrajectoryIo() { input_file_.close(); }

    std::vector<std::vector<double>> parse()
    {
        std::vector<std::vector<double>> res;
        std::string tmp;
        int linenum = 1;
        while (std::getline(input_file_, tmp, '\n')) {
            try {
                auto q = split(tmp, ",");
                res.push_back(q);
            } catch (const char *p) {
                std::cerr << "Line: " << linenum << " \"" << p << "\""
                          << " is not a number of double" << std::endl;
                break;
            }
            linenum++;
        }
        return res;
    }
    std::vector<double> split(const std::string &str, const char *delim)
    {
        std::vector<double> res;
        if ("" == str) {
            return res;
        }
        //先将要切割的字符串从string类型转换为char*类型
        char *strs = new char[str.length() + 1]; //不要忘了
        std::strcpy(strs, str.c_str());

        char *p = std::strtok(strs, delim);
        char *endp = nullptr;
        while (p) {
            double v = std::strtod(p, &endp);
            if (endp[0] != 0 && endp[0] != '\r') {
                delete[] strs;
                strs = nullptr;
                throw p;
            }
            res.push_back(v); //存入结果数组
            p = std::strtok(nullptr, delim);
        }

        if (strs) {
            delete[] strs;
            strs = nullptr;
        }

        return res;
    }

private:
    std::ifstream input_file_;
};

int exampleSpline(RpcClientPtr rpc)
{
    TrajectoryIo input("../trajs/coffee_spline.txt");
    auto traj = input.parse();
    auto traj_sz = traj.size();
    if (traj_sz == 0) {
        // 没有轨迹
        std::cerr << "No waypoints" << std::endl;
        return 0;
    }
    std::vector<double> q1 = traj[0];

    // 移动到第一个点
    printf("goto p1\n");

    auto robot_name = rpc->getRobotNames().front();

    rpc->getRobotInterface(robot_name)
        ->getMotionControl()
        ->moveJoint(q1, M_PI, M_PI, 0., 0.);
    waitArrivel(rpc->getRobotInterface(robot_name));

    for (int i = 1; i < (int)traj.size(); i++) {
        rpc->getRobotInterface(robot_name)
            ->getMotionControl()
            ->moveSpline(traj[i], 1, 1, 1);
    }
    rpc->getRobotInterface(robot_name)
        ->getMotionControl()
        ->moveSpline({}, 1, 1, 1);

    usleep(1000 * 1000 * 100);
    rpc->disconnect();

    std::cout << "end spline " << std::endl;
    return 0;
}

void printlog(int level, const char *source, int code)
{
    static const char *level_names[] = { "Critical", "Error", "Warning",
                                         "Info",     "Debug", "BackTrace" };
    fprintf(stderr, "[%s] %s - %d %s\n", level_names[level], source, code,
            arcs::error_stack::errorCode2Str(code));
}

#define LOCAL_IP "127.0.0.1"

int main(int argc, char **argv)
{
    auto rpc_cli = std::make_shared<RpcClient>();
    rpc_cli->connect(LOCAL_IP, 30004);
    rpc_cli->login("aubo", "123456");
    rpc_cli->setRequestTimeout(1000);

    auto rtde_cli = std::make_shared<RtdeClient>();
    // 接口调用: 连接到 RTDE 服务
    rtde_cli->connect(LOCAL_IP, 30010);
    // 接口调用: 登录
    rtde_cli->login("aubo", "123456");

    int topic = rtde_cli->setTopic(false, { "R1_message" }, 200, 0);
    if (topic < 0) {
        std::cout << "Set topic fail!" << std::endl;
        return -1;
    }

    rtde_cli->subscribe(topic, [](InputParser &parser) {
        arcs::common_interface::RobotMsgVector msgs;
        msgs = parser.popRobotMsgVector();
        for (size_t i = 0; i < msgs.size(); i++) {
            auto &msg = msgs[i];
            printlog(msg.level, msg.source.c_str(), msg.code);
        }
    });
    exampleSpline(rpc_cli);

    rtde_cli->removeTopic(false, topic);
    rtde_cli->logout();
    rtde_cli->disconnect();
}
