#include <cstring>
#include <fstream>
#include <math.h>

#include "aubo_sdk/rtde.h"
#include "aubo_sdk/rpc.h"
#include "aubo_sdk/script.h"

using namespace arcs::aubo_sdk;
using namespace arcs::common_interface;

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

double distance(const std::vector<double> &a, const std::vector<double> &b)
{
    double res = 0.;
    if (a.size() != b.size()) {
        return -1;
    }

    for (int i = 0; i < (int)a.size(); i++) {
        res += (a[i] - b[i]) * (a[i] - b[i]);
    }
    return sqrt(res);
}

void waitArrivel(RobotInterfacePtr impl, std::vector<double> target)
{
    while (1) {
        auto cur = impl->getRobotState()->getJointPositions();
        double dis = distance(cur, target);
        if (dis < 0.0001) {
            break;
        }
        printf("cur:%f,%f,%f,%f,%f,%f\n", cur.at(0), cur.at(1), cur.at(2),
               cur.at(3), cur.at(4), cur.at(5));
        printf("target:%f,%f,%f,%f,%f,%f\n", target.at(0), target.at(1),
               target.at(2), target.at(3), target.at(4), target.at(5));
        std::this_thread::sleep_for(std::chrono::milliseconds(1));
    }
}

template <typename T>
inline std::ostream &operator<<(std::ostream &os, const std::vector<T> &list)
{
    for (size_t i = 0; i < list.size(); i++) {
        os << list.at(i);
        if (i != (list.size() - 1)) {
            os << ",";
        }
    }
    return os;
}

class TrajectoryIo
{
public:
    TrajectoryIo(const char *filename)
    {
        input_file_.open(filename, std::ios::in);
    }
    ~TrajectoryIo() { input_file_.close(); }

    std::vector<std::vector<double>> parse()
    {
        std::vector<std::vector<double>> res;
        std::string tmp;
        int linenum = 1;
        while (std::getline(input_file_, tmp, '\n')) {
            try {
                auto q = split(tmp, ",");
                res.push_back(q);
            } catch (const char *p) {
                std::cerr << "Line: " << linenum << " \"" << p << "\""
                          << " is not a number of double" << std::endl;
                break;
            }
            linenum++;
        }
        return res;
    }
    std::vector<double> split(const std::string &str, const char *delim)
    {
        std::vector<double> res;
        if ("" == str) {
            return res;
        }
        //先将要切割的字符串从string类型转换为char*类型
        char *strs = new char[str.length() + 1]; //不要忘了
        std::strcpy(strs, str.c_str());

        char *p = std::strtok(strs, delim);
        char *endp = nullptr;
        while (p) {
            double v = std::strtod(p, &endp);
            if (endp[0] != 0 && endp[0] != '\r') {
                delete[] strs;
                strs = nullptr;
                throw p;
            }
            res.push_back(v); //存入结果数组
            p = std::strtok(nullptr, delim);
        }

        if (strs) {
            delete[] strs;
            strs = nullptr;
        }

        return res;
    }

private:
    std::ifstream input_file_;
};

#define LOCAL_IP "127.0.0.1"

int main(int argc, char **argv)
{
    auto rpc = std::make_shared<RpcClient>();
    rpc->connect(LOCAL_IP, 30004);
    rpc->login("aubo", "123456");
    TrajectoryIo input("../trajs/record6.offt");
    auto traj = input.parse();
    auto traj_sz = traj.size();
    if (traj_sz == 0) {
        // 没有轨迹
        std::cerr << "No waypoints" << std::endl;
        return 0;
    } else {
        std::cout << "Load points size: " << traj.size() << std::endl;
    }
    std::vector<double> q1 = traj[0];

    // 移动到第一个点
    printf("goto p1\n");

    auto robot_name = rpc->getRobotNames().front();
    auto mc = rpc->getRobotInterface(robot_name)->getMotionControl();

    rpc->getRuntimeMachine()->start();
    mc->moveJoint(q1, M_PI, M_PI, 0., 0.);
    waitArrivel(rpc->getRobotInterface(robot_name), q1);

    std::cout << "pathBufferAlloc " << std::endl;
    mc->pathBufferFree("rec");
    mc->pathBufferAlloc("rec", 2, traj_sz);

    size_t offset = 10;
    auto it = traj.begin();
    while (true) {
        std::cout << "pathBufferAppend " << offset << std::endl;
        mc->pathBufferAppend("rec",
                             std::vector<std::vector<double>>{ it, it + 10 });
        it += 10;
        if (offset + 10 >= traj_sz) {
            std::cout << "pathBufferAppend " << traj_sz << std::endl;
            mc->pathBufferAppend(
                "rec", std::vector<std::vector<double>>{ it, traj.end() });
            break;
        }

        offset += 10;
    }

    std::cout << "pathBufferEval " << mc->pathBufferEval("rec", {}, {}, 0.005)
              << std::endl;
    while (!mc->pathBufferValid("rec")) {
        std::cout << "pathBufferValid " << std::endl;
        std::this_thread::sleep_for(std::chrono::milliseconds(5));
    }

    mc->movePathBuffer("rec");

    rpc->disconnect();

    std::cout << "end servoj " << std::endl;
    return 0;
}
