#! /usr/bin/env python
# coding=utf-8

"""
机械臂关节运动

步骤:
第一步: 连接到 RPC 服务
第二步: 机械臂登录
第三步: 以关节运动的方式依次经过3个路点
"""

import time
import numpy as np
import pyaubo_sdk

robot_ip = "127.0.0.1"  # 服务器 IP 地址
robot_port = 30004  # 端口号
M_PI = 3.14159265358979323846
robot_rpc_client = pyaubo_sdk.RpcClient()

# 阻塞
def waitarrived(robot_name, target_q):
    """实现阻塞功能

    当前关节角与目标关节角的绝对差值小于 0.0001时，跳出循环。
    即当机械臂运动到目标路点时，程序再往下执行。

    Args:
        robot_name: 机器人的名字
        target_q: 目标关节角

    Returns:
        空
    """
    target_q_np = np.array([target_q])
    while 1:
        current_q_np = np.array(robot_rpc_client.getRobotInterface(robot_name)
                                .getRobotState().getJointPositions())  # 接口调用: 获取当前的关节角
        distance = np.linalg.norm(current_q_np - target_q_np, ord=np.Inf, axis=1)  # 当前关节角与目标关节角的绝对差值
        if distance < 0.0001:
            break
        time.sleep(1)

# 以关节运动的方式依次经过3个路点
def example_movej(robot_name):
    # 关节角，单位: 弧度
    q1 = [-2.05177, -0.400292, 1.19625, 0.0285152, 1.57033, -2.28774]
    q2 = [-0.236948, -0.465633, 1.1002, -0.00600456, 1.56824, -0.472864]
    q3 = [1.38167, -0.0115091, 1.66958, 0.107726, 1.572, 1.14576]

    # 关节运动到路点 q1
    robot_rpc_client.getRuntimeMachine().start()
    print("Moving to q1...")
    robot_rpc_client.getRobotInterface(robot_name).getMotionControl() \
        .moveJoint(q1, 180 * (M_PI / 180), 1000000 * (M_PI / 180), 0, 0)  # 接口调用: 关节运动
    waitarrived(robot_name, q1)  # 阻塞

    # 关节运动到路点 q2
    print("Moving to q2...")
    robot_rpc_client.getRobotInterface(robot_name).getMotionControl() \
        .moveJoint(q2, 180 * (M_PI / 180), 1000000 * (M_PI / 180), 0, 0)  # 接口调用: 关节运动
    waitarrived(robot_name, q2)  # 阻塞

    # 关节运动到路点 q3
    print("Moving to q3...")
    robot_rpc_client.getRobotInterface(robot_name).getMotionControl() \
        .moveJoint(q3, 180 * (M_PI / 180), 1000000 * (M_PI / 180), 0, 0)  # 接口调用: 关节运动
    waitarrived(robot_name, q3)  # 阻塞

    robot_rpc_client.getRuntimeMachine().stop()

if __name__ == '__main__':
    robot_rpc_client.connect(robot_ip, robot_port)  # 接口调用: 连接 RPC 服务
    if robot_rpc_client.hasConnected():
        print("Robot rcp_client connected successfully!")
        robot_rpc_client.login("aubo", "123456")  # 接口调用: 机械臂登录
        if robot_rpc_client.hasLogined():
            print("Robot rcp_client logined successfully!")
            robot_name = robot_rpc_client.getRobotNames()[0]  # 接口调用: 获取机器人的名字
            example_movej(robot_name)  # 以关节运动的方式依次经过3个路点
